@@ -1,12 +1,13 @@
+use crate::config;
 use crate::data;
-use clap::App;
+use crate::error;
 use serde::{Deserialize, Serialize};
 use std::default::Default;
 use structopt::StructOpt;
 pub const AUTHORS: &str = env!("CARGO_PKG_AUTHORS");
 pub const ABOUT: &str = env!("CARGO_PKG_DESCRIPTION");
 
-#[derive(StructOpt, Debug, Serialize, Deserialize)]
+#[derive(StructOpt, Debug, Default, Serialize, Deserialize)]
 #[structopt(author = AUTHORS, about = ABOUT)]
 #[serde(default, deny_unknown_fields)]
 pub struct Opt {
@@ -33,7 +34,7 @@ pub struct Opt {
     #[serde(skip_serializing, skip_deserializing)]
     pub doctor: bool,
 
-    #[structopt(short = "U", long = "short-uptime", help = "Lengthens uptime output")]
+    #[structopt(short = "U", long = "long-uptime", help = "Lengthens uptime output")]
     pub long_uptime: bool,
 
     #[structopt(short = "S", long = "long-shell", help = "Lengthens shell output")]
@@ -42,10 +43,17 @@ pub struct Opt {
     #[structopt(short = "K", long = "long-kernel", help = "Lengthens kernel output")]
     pub long_kernel: bool,
 
+    #[structopt(
+        short = "C",
+        long = "physical-cores",
+        help = "Toggles between logical and physical cores"
+    )]
+    pub physical_cores: bool,
+
     #[structopt(
         short = "s",
         long = "current-shell",
-        help = "Toggles between the current shell or the default one"
+        help = "Toggles between the current shell and the default one"
     )]
     pub current_shell: bool,
 
@@ -53,19 +61,10 @@ pub struct Opt {
         short = "t",
         long = "theme",
         case_insensitive = true,
-        help = "Specify the theme"
+        help = "Specify the name of the theme"
     )]
     pub theme: Option<String>,
 
-    #[structopt(
-        long = "export-config",
-        short = "e",
-        help = "Prints the config file to stdout",
-        conflicts_with = "doctor"
-    )]
-    #[serde(skip_serializing, skip_deserializing)]
-    pub export_config: bool,
-
     #[structopt(
         long = "list-themes",
         short = "l",
@@ -77,12 +76,21 @@ pub struct Opt {
     #[structopt(
         long = "config",
         short = "c",
-        help = "Specify the config file",
+        help = "Specify a custom path for the configuration file",
         conflicts_with = "export_config"
     )]
     #[serde(skip_serializing, skip_deserializing)]
     pub config: Option<std::path::PathBuf>,
 
+    #[structopt(
+        long = "export-config",
+        short = "e",
+        help = "Prints a template configuration file to stdout",
+        conflicts_with = "doctor"
+    )]
+    #[serde(skip_serializing, skip_deserializing)]
+    pub export_config: bool,
+
     #[structopt(
         long = "ascii-artists",
         help = "Lists the original artists of the ASCII art used by macchina"
@@ -98,27 +106,81 @@ pub struct Opt {
     pub interface: Option<String>,
 }
 
-impl Default for Opt {
-    fn default() -> Self {
-        Opt {
-            version: false,
-            doctor: false,
-            export_config: false,
-            current_shell: false,
-            long_shell: false,
-            long_uptime: false,
-            long_kernel: true,
-            list_themes: false,
-            ascii_artists: false,
-            config: None,
-            theme: None,
-            show: None,
-            interface: None,
+impl Opt {
+    pub fn parse_args(&mut self, args: Opt) {
+        if args.version {
+            self.version = true;
+        }
+
+        if args.doctor {
+            self.doctor = true;
+        }
+
+        if args.export_config {
+            self.export_config = true;
+        }
+
+        if args.current_shell {
+            self.current_shell = true;
+        }
+
+        if args.long_shell {
+            self.long_shell = true;
+        }
+
+        if args.long_uptime {
+            self.long_uptime = true;
+        }
+
+        if args.list_themes {
+            self.list_themes = true;
+        }
+
+        if args.long_kernel {
+            self.long_shell = true;
+        }
+
+        if args.physical_cores {
+            self.physical_cores = true;
+        }
+
+        if args.ascii_artists {
+            self.ascii_artists = true;
+        }
+
+        if args.config.is_some() {
+            self.config = args.config;
+        }
+
+        if args.theme.is_some() {
+            self.theme = args.theme;
+        }
+
+        if args.show.is_some() {
+            self.show = args.show;
+        }
+
+        if args.interface.is_some() {
+            self.interface = args.interface;
+        }
+    }
+
+    pub fn get_options() -> Opt {
+        let args = Opt::from_args();
+        let config_opt = match args.config {
+            Some(_) => config::read_config(&args.config.clone().unwrap()),
+            None => config::get_config(),
+        };
+
+        match config_opt {
+            Ok(mut config) => {
+                config.parse_args(args);
+                config
+            }
+            Err(e) => {
+                error::print_errors(e);
+                args
+            }
         }
     }
 }
-
-#[allow(dead_code)]
-pub fn build_cli() -> App<'static, 'static> {
-    Opt::clap()
-}
